package com.ceilfors.jenkins.plugins.jiratrigger.parameter

import com.atlassian.jira.rest.client.api.domain.Issue
import com.ceilfors.jenkins.plugins.jiratrigger.JiraTriggerException

/**
 * @author ceilfors
 */
class IssueAttributePathParameterResolver implements ParameterResolver {

    IssueAttributePathParameterMapping issueAttributePathParameterMapping

    IssueAttributePathParameterResolver(IssueAttributePathParameterMapping issueAttributePathParameterMapping) {
        this.issueAttributePathParameterMapping = issueAttributePathParameterMapping
    }

    String resolve(Issue issue) {
        resolveProperty(issue.properties, issueAttributePathParameterMapping.issueAttributePath)
    }

    /**
     * Resolves nested property from a Map.
     *
     * @param map the map which property to be resolved
     * @param property
     * @return the resolved property, null otherwise
     */
    static String resolveProperty(Map map, String property) {
        try {
            if (!property.contains('.') && !map.containsKey(property)) {
                // If property is not nested, Eval.x returns null instead of throwing NPE
                throw new JiraTriggerException(ParameterErrorCode.FAILED_TO_RESOLVE)
            }
            Eval.x(map, 'x.' + property)
        } catch (MissingPropertyException e) {
            throw new JiraTriggerException(ParameterErrorCode.FAILED_TO_RESOLVE, e)
        }
    }
}
